/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.goods.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.biz.modular.goods.entity.BizGoods;
import vip.xiaonuo.biz.modular.goods.mapper.BizGoodsMapper;
import vip.xiaonuo.biz.modular.goods.param.BizGoodsAddParam;
import vip.xiaonuo.biz.modular.goods.param.BizGoodsEditParam;
import vip.xiaonuo.biz.modular.goods.param.BizGoodsIdParam;
import vip.xiaonuo.biz.modular.goods.param.BizGoodsPageParam;
import vip.xiaonuo.biz.modular.goods.service.BizGoodsService;

import java.util.List;

/**
 * 货品管理Service接口实现类
 *
 * @author 徐超
 * @date  2025/01/02 17:44
 **/
@Service
public class BizGoodsServiceImpl extends ServiceImpl<BizGoodsMapper, BizGoods> implements BizGoodsService {

    @Override
    public Page<BizGoods> page(BizGoodsPageParam bizGoodsPageParam) {
        QueryWrapper<BizGoods> queryWrapper = new QueryWrapper<BizGoods>().checkSqlInjection();
        if(ObjectUtil.isNotEmpty(bizGoodsPageParam.getName())) {
            queryWrapper.lambda().like(BizGoods::getName, bizGoodsPageParam.getName());
        }
        if(ObjectUtil.isNotEmpty(bizGoodsPageParam.getModel())) {
            queryWrapper.lambda().like(BizGoods::getModel, bizGoodsPageParam.getModel());
        }
        if(ObjectUtil.isAllNotEmpty(bizGoodsPageParam.getSortField(), bizGoodsPageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(bizGoodsPageParam.getSortOrder());
            queryWrapper.orderBy(true, bizGoodsPageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(bizGoodsPageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(BizGoods::getId);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(BizGoodsAddParam bizGoodsAddParam) {
        BizGoods bizGoods = BeanUtil.toBean(bizGoodsAddParam, BizGoods.class);
        this.save(bizGoods);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(BizGoodsEditParam bizGoodsEditParam) {
        BizGoods bizGoods = this.queryEntity(bizGoodsEditParam.getId());
        BeanUtil.copyProperties(bizGoodsEditParam, bizGoods);
        this.updateById(bizGoods);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<BizGoodsIdParam> bizGoodsIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(bizGoodsIdParamList, BizGoodsIdParam::getId));
    }

    @Override
    public BizGoods detail(BizGoodsIdParam bizGoodsIdParam) {
        return this.queryEntity(bizGoodsIdParam.getId());
    }

    @Override
    public BizGoods queryEntity(String id) {
        BizGoods bizGoods = this.getById(id);
        if(ObjectUtil.isEmpty(bizGoods)) {
            throw new CommonException("货品管理不存在，id值为：{}", id);
        }
        return bizGoods;
    }
}
